import json
from django.shortcuts import render
from django.contrib.auth.decorators import user_passes_test
from django.shortcuts import render,redirect
from django.utils.decorators import method_decorator
from django.contrib.auth.models import  Group, Permission
from rest_framework.views import APIView
from rest_framework.response import Response
from rest_framework import status
from django.contrib.contenttypes.models import ContentType
from clmainApp.utils import getBranchId
from .serializers import *
from clmainApp.models import GROUP_CATEGORY_CHOICES
from clauthApp.models import User,USER_CATEGORY_CHOICES
from .models import *
from clauthApp.models import User
from brmainApp.models import DrData
from clhrApp.models import DrLevel
# Create your views here.


class BranchGroupsApi(APIView):
    @method_decorator(user_passes_test(lambda u: u.has_perm('BrManagement.view_group'), login_url='/auth/login/'))
    def get(self,request):
        # get all groups and all permssions of content typ app_lablel = BrManagement
        branchId = getBranchId(request)
        if not branchId : return redirect('clauthApp:home')
        groups= Group.objects.filter(group_type="BR",branch_id=branchId)
        all_users = User.objects.all().values('id','name','user_category')
        user_categories = {}
        for cat in USER_CATEGORY_CHOICES :user_categories.update({cat[0]:cat[1]})
        for u in all_users: u['user_category'] = user_categories[u['user_category']]
        #all_groups= Group.objects.all()
        serializer = BranchGroupSerializer(groups,many=True)
        # return Response({"all_groups": all_groups_serialized.data})
        return render(request,"Branch/brauthApp/branch_groups.html",
                      {"groups": serializer.data,'all_users':all_users,
                       "group_category_choices":GROUP_CATEGORY_CHOICES,
                       'activePage':'BranchGroups'})

    @method_decorator(user_passes_test(lambda u: u.has_perm('BrManagement.add_group'), login_url='/auth/login/'))
    def post(self,request):
        # post new group only without premssions
        branchId = getBranchId(request)
        if not branchId : return Response(status=status.HTTP_400_BAD_REQUEST)
        data = request.data
        data._mutable = True
        data.update({"group_type":"BR","branch":branchId,"clinic":None,"created_by":request.user.id})
        serializeobj= BranchGroupSerializer(data=data)
        if serializeobj.is_valid():
            serializeobj.save()
            return Response(serializeobj.data,status=status.HTTP_201_CREATED)
        else : return Response(serializeobj.errors,status=status.HTTP_400_BAD_REQUEST)

class GroupApi(APIView):
    def get_object(self,pk):
        try:
            return Group.objects.get(pk=pk)
        except Group.DoesNotExist:
            return None   
    
    @method_decorator(user_passes_test(lambda u: u.has_perm('BrManagement.change_permissions'), login_url='/auth/login/'))
    def get(self,request,pk):
        #get this group and it's permissions
        group=self.get_object(pk)
        #import pdb
        #pdb.set_trace()
        if group :
            if group.group_category == "1MNGMNT" : app_label_text = "BrManagement"
            elif group.group_category == "3RCP"  :app_label_text = "Reception"
            #elif group.group_category == "2CC": app_label_text = "CallCenter"
            else : return redirect("brauthApp:home")
            filtered_content_types = ContentType.objects.filter(app_label=app_label_text).order_by('id')
            content_types_ids = filtered_content_types.values_list("id",flat=True) 
            permissions=Permission.objects.filter(content_type_id__in =content_types_ids).order_by('id')
            groups_permission_ids=Group.objects.get(pk=group.id).permissions.values_list('id',flat=True)
            # content_types_managment = []
            # content_types_reception = []
            # content_types_callcenter = []
            content_types_data = []
            for ct in filtered_content_types:
                permissions_list = permissions.filter(content_type_id=ct.id).values()
                for p in permissions_list : p["is_granted"] = True if p["id"] in groups_permission_ids else False
                content_types_data.append({"model_name":ct.model,"permissions":permissions_list})
            return render(request,"Branch/brauthApp/group.html",{"group":group,"content_types_data":content_types_data,
            'activePage':'appGroups'})
            # for ct in filtered_content_types.filter(app_label="BrManagement"):#content_type
            #     permissions_list = permissions.filter(content_type_id=ct.id).values()
            #     for p in permissions_list : p["is_granted"] = True if p["id"] in groups_permission_ids else False
            #     content_types_managment.append({"model_name":ct.model,"permissions":permissions_list})
            # for ct in filtered_content_types.filter(app_label="Reception"):#content_type
            #     permissions_list = permissions.filter(content_type_id=ct.id).values()
            #     for p in permissions_list : p["is_granted"] = True if p["id"] in groups_permission_ids else False
            #     content_types_reception.append({"model_name":ct.model,"permissions":permissions_list})
            # for ct in filtered_content_types.filter(app_label="CallCenter"):#content_type
            #     permissions_list = permissions.filter(content_type_id=ct.id).values()
            #     for p in permissions_list : p["is_granted"] = True if p["id"] in groups_permission_ids else False
            #     content_types_callcenter.append({"model_name":ct.model,"permissions":permissions_list})
            #context={"model":applabel.model,"permissions":permissions,"groups_per":groups_per}
            #group_obj= GroupWithPermissionsSerializer(obj)#with permissions
            #all_Permissions= Permission.objects.filter(content_type__app_label="BrManagement").order_by('id')
            #lst = list(range(0,all_Permissions.count(),4))
            #BrManagements = []
            # for cnt in lst:
            #     BrManagement = []
            #     BrManagement.append(PermissionSerializer(all_Permissions[cnt]).data)
            #     BrManagement.append(PermissionSerializer(all_Permissions[cnt+1]).data)
            #     BrManagement.append(PermissionSerializer(all_Permissions[cnt+2]).data)
            #     BrManagement.append(PermissionSerializer(all_Permissions[cnt+3]).data)
            #     BrManagements.append(BrManagement)
            #return Response({"group":group_obj.data,"all_permissions": BrManagements},status=status.HTTP_200_OK)
            # return render(request,"Branch/brauthApp/group.html",{"group":group,"content_types_data":content_types_data,
            # # "content_types_managment": content_types_managment,"content_types_reception" :content_types_reception,
            # # "content_types_callcenter" :content_types_callcenter,
            # 'activePage':'appGroups'})
        

    # update group permissions to with new list
    @method_decorator(user_passes_test(lambda u: u.has_perm('BrManagement.add_group'), login_url='/auth/login/'))
    def post(self,request,pk):
        try :
            group=self.get_object(pk)
            data = request.data
            checked_permissions_json = data['checked_permissions'] 
            checked_permissions = json.loads(checked_permissions_json)
            group.permissions.clear()
            for perm in checked_permissions: group.permissions.add(perm)
            return Response(status=status.HTTP_200_OK)
        except : Response(status=status.HTTP_400_BAD_REQUEST) 
    # update group permissions to with new list
    @method_decorator(user_passes_test(lambda u: u.has_perm('BrManagement.change_group'), login_url='/auth/login/'))
    def put(self,request,pk):
        obj=self.get_object(pk)
        data=request.data
        serializeobj= BranchGroupSerializer(obj,data,partial=True)
        if serializeobj.is_valid():
            serializeobj.save()
            return Response(status=status.HTTP_200_OK)
        else : Response(serializeobj.errors,status=status.HTTP_400_BAD_REQUEST)    
        '''
        #isgroupEdit=request.POST.get("isgroupEdit")
        #permissionslist=request.POST.getlist("permissions[]")
        
        if isgroupEdit == '1' :
            serializeobj= GroupSerializer(obj,data)
            if serializeobj.is_valid():
                serializeobj.save()
                return Response(status=status.HTTP_200_OK)
            else : Response(serializeobj.errors,status=status.HTTP_400_BAD_REQUEST)  
        else:
            serializeobj= GroupSerializer(obj,data)
            if serializeobj.is_valid():
                serializeobj.save()
                obj.permissions.clear()
                for perm in permissionslist:
                    obj.permissions.add(perm)
                return Response(serializeobj.data,status=status.HTTP_200_OK)
            else :return Response(serializeobj.errors,status=status.HTTP_400_BAD_REQUEST)
        return Response(serializeobj.errors,status=status.HTTP_400_BAD_REQUEST) 
        '''

 
    @method_decorator(user_passes_test(lambda u: u.has_perm('BrManagement.delete_group'), login_url='/auth/login/'))
    def delete(self,request,pk):
        obj=self.get_object(pk)
        if obj :
            if obj.user_set.all().count() == 0:
                try:
                    obj.delete()
                    return Response(status=status.HTTP_204_NO_CONTENT)
                except Exception as err_msg: 
                    return Response(data ="{}".format(str(err_msg)),status=status.HTTP_405_METHOD_NOT_ALLOWED)
            else:
                return Response(data ="يرجي حذف البيانات المرتبطة أولا لحذف هذا العنصر !!!",status=status.HTTP_405_METHOD_NOT_ALLOWED)
        else : return Response(status=status.HTTP_404_NOT_FOUND)


from django.db import transaction

class BranchGroupUsersApi(APIView):
    def get_object(self,pk):
        try:
            return Group.objects.get(pk=pk)
        except Group.DoesNotExist:
            return None   

    @method_decorator(user_passes_test(lambda u: u.has_perm('BrManagement.add_users_to_group'), login_url='/auth/login/'))
    def get(self,request,group_id):
        #groups= Group.objects.all()
        group=self.get_object(group_id)
        if not group :return Response(status=status.HTTP_400_BAD_REQUEST)
        users_ids=group.user_set.all().values_list('id',flat=True)#Users for selected Group 
        #serializer = UserMiniSerializer(users,many=True)
        return Response(users_ids,status=status.HTTP_200_OK)
        #groups_serialized = GroupSerializer(groups,many=True)
        #users_serialized = UserSerializer(users,many=True)    


    @method_decorator(user_passes_test(lambda u: u.has_perm('BrManagement.add_users_to_group'), login_url='/auth/login/'))
    def post(self,request,group_id):
        branchId = getBranchId(request)
        if not branchId : return redirect('clauthApp:home')
        group=self.get_object(group_id)
        if not group :return Response(status=status.HTTP_400_BAD_REQUEST)
        data = request.data
        selected_users = data.getlist('users[]')
        #group_users = []
        try :
            group.user_set.clear()# clear group users 
            group.user_set.add(*selected_users) # astrisk used for list  user_set.add(user_id) Or user_set.add(*users_id_list)
            # add DrData object for new doctors
            #doctors_data_objects = User.objects.filter(user_category="2DR",groups__branch_id=branchId,is_active=True).values("id","number_1")
            #branch_doctors_ids_from_branch_group = doctors_data_objects.values_list("id",flat=True)
            #doctors_data_branch = DrData.objects.filter(branch_id=branchId)
            #doctors_data_branch__ids = doctors_data_branch.values_list('dr_id',flat=True)
            # Get the ids of doctors in the branch group but not in the DrData table for this branch
            #doctors_to_add = set(branch_doctors_ids_from_branch_group)
            #doctors_to_add = set(branch_doctors_ids_from_branch_group) - set(doctors_data_branch__ids)

            # Get the ids of doctors in the DrData table for this branch but not in the branch group
            #doctors_to_delete = set(doctors_data_branch__ids) - set(branch_doctors_ids_from_branch_group)
            
            #current_default_dr_level_id = None
            #if doctors_to_add : current_default_dr_level_id = DrLevel.objects.filter(branch_id=branchId,is_default=True).first().id
            # Perform both actions inside a transaction for atomicity
            #with transaction.atomic():
                # Add new DrData objects for doctors in doctors_to_add
                # for doctor_id in doctors_to_add:DrData.objects.create(branch_id=branchId, dr_id=doctor_id,
                #                                                       dr_level_id=current_default_dr_level_id,
                #                                                       salary_phone_number=doctors_data_objects.filter(id=doctor_id).first()["number_1"],
                #                                                       is_active=True)
                # To Active Again ( have Dr Object and in group )
                #doctors_to_make_active_ids = set(doctors_data_branch__ids) & set(branch_doctors_ids_from_branch_group)
                #DrData.objects.filter(dr_id__in=doctors_to_make_active_ids).update(is_active=True)
                # Delete DrData objects for doctors in doctors_to_delete change to not active 
                #DrData.objects.filter(dr_id__in=doctors_to_delete).update(is_active=False)

            return Response(status=status.HTTP_201_CREATED)
        except Exception as err_msg: return Response("{} \n خطأ أثناء إضافة الأطباء للمجموعة \n برجاء التواصل مع الدعم لحل المشكلة فى اقرب وقت ".format(str(err_msg)),status=status.HTTP_400_BAD_REQUEST)
        #except Exception as err_msg: return Response("{} \n برجاء التأكد من اضافة مستويات الأطباء أولا".format(str(err_msg)),status=status.HTTP_400_BAD_REQUEST)


